import os
from math import prod

import cbutil as cb


def mesa_pd(text_file: str):
    line_wise = cb.read_file_line_wise(text_file)

    fields = {key: value for key, value in
              cb.process_linewise(cb.mesa_pd_text, line_wise)}

    time_stamp = cb.util.file_time_to_sec(text_file)
    yield cb.DataPoint(
        "MESA_PD_KernelBenchmark",
        time=time_stamp,
        fields=fields, tags=dict()).asdict()


def mesa_pd_sqlite(db_file: str, *,
                   tag_key={"mpi_num_processes",
                            "omp_max_threads", "walberla_git", }
                   ):

    with cb.sqlite_context(db_file) as connection:
        runs = cb.query_complete_table(connection, "runs")
        for run in runs.fetchall():
            time = cb.util.time_conversion(run["timestamp"])
            runid = run["runId"]
            tags = {k: run[k] for k in run.keys() if k in tag_key}
            fields = dict()
            for timing in cb.build_iterate_query(connection,
                                                 from_table="timingPool",
                                                 where_args=["runId", runid]):
                fields.update(
                    {timing["sweep"]: timing["average"]*timing["count"]})

            yield cb.DataPoint("MESA_PD_KernelBenchmark",
                               time=time,
                               fields=fields,
                               tags=tags).asdict()


def _iter_runs(runs, *, measurement_name):
    no_tag_keys = {'vtkWriteFrequency',
                   'remainingTimeLoggerFrequency',
                   'executable',
                   'compile_flags',
                   'id', }
    field_names = {"mlupsPerProcess"}
    time_key = "timestamp"

    for run in runs:
        dp = cb.data_point_factory(run,
                                   time_key=time_key,
                                   measurement_name=measurement_name,
                                   field_keys=field_names,
                                   no_tag_keys=no_tag_keys)
        dp.tags.update(
            {"mpi_num_processes": prod(
                [int(run[key]) for key in run.keys() if key.startswith("block")])}
        )

        yield dp.asdict()


def _get_arch(db_file: str) -> str:
    arch, *_ = db_file.split("_")
    return arch.upper()


def uniformgrid_sqlite(db_file: str):
    measurement_name = f"UniformGrid{_get_arch(db_file)}"
    yield from _iter_runs(cb.iterate_all_tables(db_file),
                          measurement_name=measurement_name)


def pfac_csv(csv_file: str):
    measurement_name = "PhaseFieldAllenCahn"
    if (arch := _get_arch(csv_file)) != "CPU":
        measurement_name += arch
    yield from _iter_runs(cb.iterate_csv(csv_file),
                          measurement_name=measurement_name)


def energy_json(json_file: str):
    measurement_name = os.path.splitext(os.path.basename(json_file))[0]
    result_dict = cb.json2dict(json_file)
    base_keys = ['ENERGY', 'ENERGY', 'Metric']
    metric_dict = cb.get_from_nested_dict(result_dict, base_keys)
    inner_dict = 'Values'

    def search_for_metric(metric: str, unit: str):
        def get_metric_key():
            return f'{metric} PKG [{unit}]'
        key = get_metric_key()
        try:
            ret = metric_dict[key]
        except KeyError:
            ret = metric_dict[f'{metric} [{unit}]']
        return {key: ret[inner_dict][0]}

    energy = search_for_metric('Energy', 'J')
    power = search_for_metric('Power', 'W')

    time = cb.file_time_to_sec(json_file)
    yield cb.DataPoint(measurement=measurement_name,
                       time=time,
                       fields={**energy, **power},
                       tags=dict(),)
